searchNodes=[{"ref":"Vault.html","title":"Vault","type":"module","doc":"The main module for configuring and interacting with HashiCorp&#39;s Vault. When possible, it tries to emulate the CLI, with read, write, list and delete methods. An additional request method is provided when you need further flexibility. API Preview vault = Vault.new([ engine: Vault.Engine.KVV2, auth: Vault.Auth.UserPass ]) |&gt; Vault.auth(%{username: &quot;username&quot;, password: &quot;password&quot;}) {:ok, db_pass} = Vault.read(vault, &quot;secret/path/to/password&quot;) {:ok, %{&quot;version&quot; =&gt; 1 }} = Vault.write(vault, &quot;secret/path/to/creds&quot;, %{secret: &quot;secrets!&quot;}) Flexibility Hashicorp&#39;s Vault is highly configurable. Rather than cover every possible option, this library strives to be flexible and adaptable. Auth backends, Secret Engines, and Http clients, and JSON parsers are all replacable, and each behaviour asks for a minimal behaviour contract. HTTP Adapters The following HTTP Adapters are provided: Tesla with Vault.HTTP.Tesla Can be configured to use :hackney, :ibrowse, or :httpc (and in turn, plays nice with HTTPoison, and HTTPotion) JSON Adapters Most JSON libraries provide the same methods, so no default adapter is needed. You can use Jason, JSX, Poison, or whatever encoder you want. Defaults to Jason if present, falling back to Poison if present, ultimately falling back to nil. See Vault.JSON.Adapter for the full behaviour interface. Auth Adapters Adapters have been provided for the following auth backends: AppRole with Vault.Auth.Approle Azure with Vault.Auth.Azure GitHub with Vault.Auth.Github GoogleCloud with with Vault.Auth.GoogleCloud JWT with Vault.Auth.JWT Kubernetes with Vault.Auth.Kubernetes LDAP with Vault.Auth.LDAP UserPass with Vault.Auth.UserPass Token with Vault.Auth.Token In addition to the above, a generic backend is also provided (Vault.Auth.Generic). If support for auth provider is missing, you can still get up and running quickly, without writing a new adapter. Secret Engines Most of Vault&#39;s Secret Engines use a replacable API. The Vault.Engine.Generic adapter should handle most use cases for secret fetching. This is also the default value. Vault&#39;s KV version 2 broke away from the standard REST convention. So KV has been given its own adapter: Key/Value v1 with Vault.Engine.KVV1 v2 with Vault.Engine.KVV1 Additional request methods The core library only handles the basics around secret fetching. If you need to access additional API endpoints, this library also provides a Vault.request method. This should allow you to tap into the complete vault REST API, while still benefiting from token control, JSON parsing, and other HTTP client nicities. Setup and Usage First, ensure that adapter dependancies are part of your dependancies and applications: def application do [ extra_applications: [:logger, :tesla, :hackney,] # or :ibrowse mod: {MyApp.Application, []} ] end def deps do [ # tesla, required for Vault.HTTP.Tesla {:tesla, &quot;~&gt; 1.0.0&quot;, }, # pick your HTTP client - ibrowse, hackney, or if you&#39;re feeling bold, httpc. {:ibrowse, &quot;~&gt; 4.4.0&quot;, }, {:hackney, &quot;~&gt; 1.6&quot;, }, {:mint, &quot;~&gt; 1.0&quot;}, {:castore, &quot;~&gt; 1.1&quot;}, # mint requires castore # Pick your json parser {:jason, &quot;&gt;= 1.0.0&quot;, }, {:poison, &quot;~&gt; 3.0&quot;, }, ] end Example usage: vault = Vault.new([ engine: Vault.Engine.KVV2, auth: Vault.Auth.UserPass, credentials: %{username: &quot;username&quot;, password: &quot;password&quot;} http: Vault.Http.Tesla, # Note - `:http_options` are specific to the http adapter. http_options: [{ ssl_options: [certfile: &quot;certs/client.crt&quot;] }] host: &quot;https://my.vault.pizza&quot; ]) |&gt; Vault.auth() {:ok, db_pass} = Vault.read(vault, &quot;secret/path/to/password&quot;) {:ok, aws_creds} = Vault.read(vault, &quot;secret/path/to/creds&quot;) You can configure the client up front, or change configuration dynamically. vault = Vault.new() # use defaults, or application configuration |&gt; Vault.set_auth(Vault.Auth.Approle) |&gt; Vault.auth(%{role_id: &quot;role_id&quot;, secret_id: &quot;secret_id&quot;}) {:ok, db_pass} = Vault.read(vault, &quot;secret/path/to/password&quot;) vault = Vault.set_engine(Vault.Engine.KVV2) // switch to versioned secrets {:ok, db_pass} = Vault.write(vault, &quot;kv/path/to/password&quot;, %{ password: &quot;db_pass&quot; })"},{"ref":"Vault.html#auth/2","title":"Vault.auth/2","type":"function","doc":"Authenticate against the configured auth backend. Examples A successful authentication returns a client containing a valid token, as well as the expiration time for the token. Perform this operation before reading or writing secrets. Errors from vault are returned as a list of strings. Uses pre-configured credentials if provided. Passed in credentials will override existing credentials. {:ok, vault} = Vault.set_credentials(vault, %{username: &quot;UserN4me&quot;, password: &quot;P@55w0rd&quot;}) {:error, [&quot;Missing Credentials, username and password are required&quot;]} = Vault.set_credentials(vault, %{username: &quot;whoops&quot;})"},{"ref":"Vault.html#delete/3","title":"Vault.delete/3","type":"function","doc":"Delete a secret from the configured secret engine. Examples Returns the response from vault, which is typically an empty map. See Secret Engine Adapter options for further configuration. {:ok, %{} }} = Vault.delete(vault,&quot;secret/path/to/write&quot;) {:error, [&quot;Key not found&quot;]} = Vault.list(vault,&quot;secret/bad/path/&quot;)"},{"ref":"Vault.html#list/3","title":"Vault.list/3","type":"function","doc":"List secret keys available at a certain path. Examples Path should end with a trailing slash. Provided adapters returns the values on the data key from vault, if present. See Secret Engine adapter details for additional configuration, such as returning the full response. Errors from vault are returned as a list of strings. {:ok, %{ &quot;keys&quot; =&gt; [&quot;some/&quot;, &quot;paths&quot;, &quot;returned&quot;] }} = Vault.list(vault,&quot;secret/path/to/write&quot;) {:error, [&quot;Unauthorized&quot;]} = Vault.list(vault,&quot;secret/bad/path/&quot;)"},{"ref":"Vault.html#new/1","title":"Vault.new/1","type":"function","doc":"Create a new client. Optionally provide a keyword list or map of options for configuration. Examples Return a default Vault client: vault = Vault.new() Return a fully initialized Vault Client: vault = Vault.new(%{ http: Vault.HTTP.Tesla, host: myvault.instance.com, auth: Vault.Auth.JWT, auth_path: &#39;jwt&#39;, engine: Vault.Engine.Generic, token: &quot;abc123&quot;, token_expires_at: NaiveDateTime.utc_now() |&gt; NaiveDateTime.add(30, :second), credentials: %{role_id: &quot;dev-role&quot;, jwt: &quot;averylongstringoflettersandnumbers...&quot;} }) Options The following options can be provided as part of the :vault application config, or as a Keyword List or Map of options. Runtime configuration will always take precedence. :auth - Module for your Auth adapter. :auth_path - Path to use for your auth adapter. Provided adapters have their own default paths. Check your adapter for details. :engine Module for your Secret Engine adapter. Defaults to Vault.Engine.Generic. :host - host of your vault instance. Should contain the port, if needed. Should not contain a trailing slash. Defaults to System.get_env(&quot;VAULT_ADDR&quot;). :http - Module for your http adapter. Defaults to Vault.HTTP.Tesla when :tesla is present. :http_options - A keyword list of options to your HTTP adapter. :token - A vault token. :token_expires_at A NaiveDateTime instance that represents when the token expires, in utc. :credentials - The credentials to use when authenticating with your Auth adapter."},{"ref":"Vault.html#read/3","title":"Vault.read/3","type":"function","doc":"Read a secret from the configured secret engine. Examples Provided adapters return the values on the data key from vault, if present. See Secret Engine adapter details for additional configuration, such as returning the full response. Errors from vault are returned as a list of strings. {:ok, %{ password: &quot;value&quot; }} = Vault.write(vault,&quot;secret/path/to/read&quot;) {:error, [&quot;Unauthorized&quot;]} = Vault.read(vault,&quot;secret/bad/path&quot;)"},{"ref":"Vault.html#request/4","title":"Vault.request/4","type":"function","doc":"Make an HTTP request against your vault instance, with the current vault token. Examples This library doesn&#39;t cover every vault API, but this can help fill some of the gaps, and removing some boilerplate around token management, and JSON parsing. It can also be handy for renewing dynamic secrets, if you&#39;re using the AWS Secret backend. Requests can take the following options a Keyword List. options: :query_params - a keyword list of query params for the request. Do not include query params on the path. :body - Map. The body for the request :headers - Keyword list. The headers for the request :version - String. The vault api version - defaults to &quot;v1&quot; General Example Here&#39;s a genneric example for making a request: vault = Vault.new( http: Vault.HTTP.Tesla, host: &quot;http://localhost&quot;, token: &quot;token&quot; token_expires_in: NaiveDateTime.utc_now() ) Vault.request(vault, :post, &quot;path/to/call&quot;, [ body: %{ &quot;foo&quot; =&gt; &quot;bar&quot;}]) # POST to http://localhost/v1/path/to/call # with headers: {&quot;X-Vault-Token&quot;, &quot;token&quot;} # and a JSON payload of: &quot;{ &#39;foo&#39;: &#39;bar&#39;}&quot; AWS lease renewal A quick example of renewing a lease. vault = Vault.new( http: Vault.HTTP.Tesla, host: &quot;http://localhost&quot;, token: &quot;token&quot; token_expires_in: NaiveDateTime.utc_now() ) body = %{lease_id: lease, increment: increment} {:ok, response} = Vault.request(vault, request(:put, &quot;sys/leases/renew&quot;, [body: body])"},{"ref":"Vault.html#set_auth/2","title":"Vault.set_auth/2","type":"function","doc":"Set the backend to use for authenticating the client. Examples The auth backend should be a module that meets the Vault.Auth.Adapter behaviour. vault = Vault.set_auth(vault, Vault.Auth.Approle)"},{"ref":"Vault.html#set_auth_path/2","title":"Vault.set_auth_path/2","type":"function","doc":"Set the path used when logging in with your auth adapter. Examples Auth backends can be mounted at any path on /auth/. If left unset, the auth adapter may provide a default, eg userpass. See your Auth adapter for details. vault = Vault.set_auth_path(vault, &quot;auth-path&quot;)"},{"ref":"Vault.html#set_credentials/2","title":"Vault.set_credentials/2","type":"function","doc":"Sets the login credentials for this client. Examples vault = Vault.set_credentials(vault, %{username: &quot;UserN4me&quot;, password: &quot;P@55w0rd&quot;})"},{"ref":"Vault.html#set_engine/2","title":"Vault.set_engine/2","type":"function","doc":"Set the secret engine for the client. Examples The secret engine should be a module that meets the Vault.Engine.Adapter behaviour. vault = Vault.set_engine(vault, Vault.Engine.KVV2)"},{"ref":"Vault.html#set_host/2","title":"Vault.set_host/2","type":"function","doc":"Set the host of your vault instance. Examples The host can be fetched from anywhere, as long as it&#39;s a string. vault = Vault.set_host(vault, System.get_env(&quot;VAULT_ADDR&quot;)) The port should be provided if needed, along with the protocol. vault = Vault.set_host(vault, &quot;https://my-vault.host.com:12345&quot;)"},{"ref":"Vault.html#set_http/2","title":"Vault.set_http/2","type":"function","doc":"Set the http module used to make API calls. Examples Should be a module that meets the Vault.HTTP.Adapter behaviour. vault = Vault.set_http(vault, Vault.HTTP.Tesla)"},{"ref":"Vault.html#token_expired?/1","title":"Vault.token_expired?/1","type":"function","doc":"Check if the current token is still valid. Examples Returns true if the current time is later than the expiration date, otherwise false. true = Vault.token_expired?(vault)"},{"ref":"Vault.html#token_expires_at/1","title":"Vault.token_expires_at/1","type":"function","doc":"Get a NaiveDateTime struct, in UTC, for when the current token will expire. Examples Expiration time is generated from the current time on the current server. ~N[2018-11-25 16:30:30.177731] = Vault.token_expires_at(vault)"},{"ref":"Vault.html#write/4","title":"Vault.write/4","type":"function","doc":"Write a secret to the configured secret engine. Examples Provided adapters returns the values on the data key from vault, if present. See Secret Engine adapter details for additional configuration, such as returning the full response. Errors from vault are returned as a list of strings. {:ok, %{ version: 1 }} = Vault.write(vault,&quot;secret/path/to/write&quot;, %{ secret: &quot;value&quot;}) {:error, [&quot;Unauthorized&quot;]} = Vault.write(vault,&quot;secret/bad/path&quot;, %{ secret: &quot;value&quot;})"},{"ref":"Vault.html#t:auth/0","title":"Vault.auth/0","type":"type","doc":""},{"ref":"Vault.html#t:auth_path/0","title":"Vault.auth_path/0","type":"type","doc":""},{"ref":"Vault.html#t:credentials/0","title":"Vault.credentials/0","type":"type","doc":""},{"ref":"Vault.html#t:engine/0","title":"Vault.engine/0","type":"type","doc":""},{"ref":"Vault.html#t:host/0","title":"Vault.host/0","type":"type","doc":""},{"ref":"Vault.html#t:http/0","title":"Vault.http/0","type":"type","doc":""},{"ref":"Vault.html#t:json/0","title":"Vault.json/0","type":"type","doc":""},{"ref":"Vault.html#t:method/0","title":"Vault.method/0","type":"type","doc":""},{"ref":"Vault.html#t:options/0","title":"Vault.options/0","type":"type","doc":""},{"ref":"Vault.html#t:t/0","title":"Vault.t/0","type":"type","doc":""},{"ref":"Vault.html#t:token/0","title":"Vault.token/0","type":"type","doc":""},{"ref":"Vault.html#t:token_expires_at/0","title":"Vault.token_expires_at/0","type":"type","doc":""},{"ref":"Vault.Auth.Adapter.html","title":"Vault.Auth.Adapter","type":"behaviour","doc":"Adapter interface for authenticating with vault. Writing your own adapter Auth adapters are pretty simple. You build a url, map the parameters, and grab the response. Feel free to use the provided Vault.HTTP module to make http requests against your vault instance. In most cases, you&#39;ll end up sending a POST to auth/SOME_BACKEND/login, and pass the parameters along as a body. Below, you&#39;ll find a starting template for your own adapter. If you&#39;re writing an official implementation, check the Docs link below for the spec. Vault Auth Method Docs defmodule Vault.Auth.MyAuth do @behaviour Vault.Auth.Adapter @impl true def login(%Vault{} = vault, %{username: _, password: _} = params) do headers = [ {&quot;Content-Type&quot;, &quot;application/json&quot;}, {&quot;Accept&quot;, &quot;application/json&quot;} ] url = &quot;auth/MY_NEW_AUTH/login&quot; request_options = [body: %{ password: password }, headers: headers] with {:ok, response} &lt;- Vault.HTTP.post(vault, url, request_options) do case response do %{&quot;errors&quot; =&gt; messages} -&gt; {:error, messages} %{&quot;auth&quot; =&gt; %{&quot;client_token&quot; =&gt; token, &quot;lease_duration&quot; =&gt; ttl}} -&gt; {:ok, token, ttl} otherwise -&gt; {:error, [&quot;Unexpected response from vault.&quot;, inspect(otherwise)]} end else {:error, response} -&gt; {:error, [&quot;Http adapter error&quot;, inspect(response)]} end end def login(%Vault{http: http, host: host}, _params), do: {:error, [&quot;Missing params! Username and password are required.&quot;]} end"},{"ref":"Vault.Auth.Adapter.html#c:login/2","title":"Vault.Auth.Adapter.login/2","type":"callback","doc":""},{"ref":"Vault.Auth.Adapter.html#t:errors/0","title":"Vault.Auth.Adapter.errors/0","type":"type","doc":""},{"ref":"Vault.Auth.Adapter.html#t:params/0","title":"Vault.Auth.Adapter.params/0","type":"type","doc":""},{"ref":"Vault.Auth.Adapter.html#t:response/0","title":"Vault.Auth.Adapter.response/0","type":"type","doc":""},{"ref":"Vault.Auth.Adapter.html#t:token/0","title":"Vault.Auth.Adapter.token/0","type":"type","doc":""},{"ref":"Vault.Auth.Adapter.html#t:ttl/0","title":"Vault.Auth.Adapter.ttl/0","type":"type","doc":""},{"ref":"Vault.Auth.Adapter.html#t:vault/0","title":"Vault.Auth.Adapter.vault/0","type":"type","doc":""},{"ref":"Vault.Auth.Approle.html","title":"Vault.Auth.Approle","type":"module","doc":"Approle Auth Adapter. Vault Docs"},{"ref":"Vault.Auth.Approle.html#login/2","title":"Vault.Auth.Approle.login/2","type":"function","doc":"login with a role id, and secret id. Defaults the auth path to approle Examples # Atom Map {:ok, token, ttl } = Vault.Auth.Approle.login(%{role_id: role_id, secret_id: secret_id}) # String map {:ok, token, ttl } = Vault.Auth.Approle.login(%{&quot;role_id&quot; =&gt; role_id, &quot;secret_id&quot; =&gt; secret_id})"},{"ref":"Vault.Auth.Azure.html","title":"Vault.Auth.Azure","type":"module","doc":"Azure Auth Adapter. Vault Docs"},{"ref":"Vault.Auth.Azure.html#login/2","title":"Vault.Auth.Azure.login/2","type":"function","doc":"Login with your Azure role and JWT. Defaults the auth path to azure Examples # Atom map {:ok, token, ttl} = Vault.Auth.Azure.login(vault, %{role: &quot;my-role&quot;, jwt: &quot;my-jwt&quot;}) # String map {:ok, token, ttl} = Vault.Auth.Azure.login(vault, %{&quot;role&quot; =&gt; &quot;my-role&quot;, &quot;jwt&quot; =&gt; &quot;my-jwt&quot;})"},{"ref":"Vault.Auth.Generic.html","title":"Vault.Auth.Generic","type":"module","doc":"A Generic Auth Adapter. An alternative to writing your own adapter."},{"ref":"Vault.Auth.Generic.html#login/2","title":"Vault.Auth.Generic.login/2","type":"function","doc":"Authenticate with a custom auth method. Provide options for the request, and how to parse the response. Examples request defines parameters for the request to vault path- the path for authentication, after &quot;auth&quot; If you want to authenticate against https://myvault.com/v1/auth/jwt/login, then the path would be jwt/login method- one of :get, :post, :put, :patch, :delete, defaults to :post body- any params needed to login. Defaults to %{} response defines parameters for parsing the response. token_path - a list of properties that describe the JSON path to a token. Defaults to [&quot;auth&quot;, &quot;client_token&quot;] ttl_path - a list of properties that describe the JSON path to the ttl, or lease duration. Defaults to [&quot;auth&quot;, &quot;lease_duration&quot;] The following would provide a minimal adapter for the JWT backend: {:ok, token, ttl} = Vault.Auth.Generic.login(vault, %{ request: %{ path: &quot;/jwt/login&quot;, body: %{role: &quot;my-role&quot;, jwt: &quot;my-jwt&quot; }, } }) Here&#39;s the above example as part of the full Vault client flow. On success, it returns an authenticated vault client. vault = Vault.new([ auth: Vault.Auth.Generic, http: Vault.HTTP.Tesla, engine: Vault.KVV2 ]) {:ok, vault} = Vault.auth(vault, %{ request: %{ path: &quot;/jwt/login&quot;, body: %{role: &quot;my-role&quot;, jwt: &quot;my-jwt&quot; }, } }) Here&#39;s a more explicit example, with every option configured. vault = Vault.new([ auth: Vault.Auth.Generic, http: Vault.HTTP.Tesla, engine: Vault.KVV2 ]) {:ok, vault} = Vault.auth(vault, %{ request: path: &quot;/jwt/login&quot;, method: :post, body: %{role: &quot;my-role&quot;, jwt: &quot;my-jwt&quot; }, response: %{ token: [&quot;auth&quot;, &quot;client_token&quot;], ttl: [&quot;auth&quot;, &quot;lease_duration&quot;] } })"},{"ref":"Vault.Auth.Generic.html#t:params/0","title":"Vault.Auth.Generic.params/0","type":"type","doc":""},{"ref":"Vault.Auth.Generic.html#t:request/0","title":"Vault.Auth.Generic.request/0","type":"type","doc":""},{"ref":"Vault.Auth.Generic.html#t:response/0","title":"Vault.Auth.Generic.response/0","type":"type","doc":""},{"ref":"Vault.Auth.Github.html","title":"Vault.Auth.Github","type":"module","doc":"Github Auth Adapter Vault Docs"},{"ref":"Vault.Auth.Github.html#login/2","title":"Vault.Auth.Github.login/2","type":"function","doc":"Log in with a github access token. Defaults the auth path to github Examples # Atom map {:ok, token, ttl} = Vault.Auth.Github.login(vault, %{token: access_token}) # String map {:ok, token, ttl} = Vault.Auth.Github.login(%{&quot;token&quot; =&gt; access_token })"},{"ref":"Vault.Auth.GoogleCloud.html","title":"Vault.Auth.GoogleCloud","type":"module","doc":"Google Cloud Auth Adapter. Vault Docs"},{"ref":"Vault.Auth.GoogleCloud.html#login/2","title":"Vault.Auth.GoogleCloud.login/2","type":"function","doc":"Login with your Google Cloud role and JWT. Defaults the auth path to gcp Examples # Atom map {:ok, token, ttl} = Vault.Auth.GoogleCloud.login(vault, %{role: &quot;my-role&quot;, jwt: &quot;my-jwt&quot;}) # String Map {:ok, token, ttl} = Vault.Auth.GoogleCloud.login(vault, %{&quot;role&quot; =&gt; &quot;my-role&quot;, &quot;jwt&quot; =&gt; &quot;my-jwt&quot;})"},{"ref":"Vault.Auth.JWT.html","title":"Vault.Auth.JWT","type":"module","doc":"JWT Auth Adapter. Vault Docs"},{"ref":"Vault.Auth.JWT.html#login/2","title":"Vault.Auth.JWT.login/2","type":"function","doc":"Login with a JWT role and jwt token. Defaults the auth path to jwt Examples # Atom Map {:ok, token, ttl} = Vault.Auth.JWT.login(vault, %{role: &quot;dev-role&quot;, jwt: &quot;my-jwt&quot;}) # String Map {:ok, token, ttl} = Vault.Auth.JWT.login(vault, %{&quot;role&quot; =&gt; &quot;my-role&quot;, &quot;jwt&quot; =&gt; &quot;my-jwt&quot;})"},{"ref":"Vault.Auth.Kubernetes.html","title":"Vault.Auth.Kubernetes","type":"module","doc":"Kubernetes Auth Adapter. Vault Docs"},{"ref":"Vault.Auth.Kubernetes.html#login/2","title":"Vault.Auth.Kubernetes.login/2","type":"function","doc":"Login with your Kubernetes role and JWT. Defaults the auth path to kubernetes Examples # Atom map {:ok, token, ttl} = Vault.Auth.Kubernetes.login(vault, %{role: &quot;my-role&quot;, jwt: &quot;my-jwt&quot;}) # String Map {:ok, token, ttl} = Vault.Auth.Kubernetes.login(vault, %{&quot;role&quot; =&gt; &quot;my-role&quot;, &quot;jwt&quot; =&gt; &quot;my-jwt&quot;})"},{"ref":"Vault.Auth.LDAP.html","title":"Vault.Auth.LDAP","type":"module","doc":"LDAP Auth Adapter Vault Docs"},{"ref":"Vault.Auth.LDAP.html#login/2","title":"Vault.Auth.LDAP.login/2","type":"function","doc":"Login with your LDAP username and password. Defaults the auth path to ldap Examples # Atom map {:ok, token, ttl} = Vault.Auth.LDAP.login(%{ username: &quot;username&quot;, password: &quot;password&quot; }) # String map {:ok, token, ttl} = Vault.Auth.LDAP.login(%{ &quot;username&quot; =&gt; &quot;username&quot;, &quot;password&quot; =&gt; &quot;password&quot; })"},{"ref":"Vault.Auth.Token.html","title":"Vault.Auth.Token","type":"module","doc":"Token Auth Adapter. Checks a provided token for validity, and saves if valid. Useful for local dev, or writing a CLI that uses the .vault-token file in the home directory. Vault Docs"},{"ref":"Vault.Auth.Token.html#login/2","title":"Vault.Auth.Token.login/2","type":"function","doc":"Log in with an existing vault token. Auth path not required. Examples # Atom map {:ok, token, ttl} = Vault.Auth.Token.login(vault, %{token: local_token}) # String map {:ok, token, ttl} = Vault.Auth.Token.login(vault, %{&quot;token&quot; =&gt; local_token})"},{"ref":"Vault.Auth.Token.html#url/0","title":"Vault.Auth.Token.url/0","type":"function","doc":""},{"ref":"Vault.Auth.UserPass.html","title":"Vault.Auth.UserPass","type":"module","doc":"Userpass Auth Adapter https://www.vaultproject.io/docs/auth/userpass.html"},{"ref":"Vault.Auth.UserPass.html#login/2","title":"Vault.Auth.UserPass.login/2","type":"function","doc":"Login with a username and password. Defaults the auth path to userpass Examples # Atom map {:ok, token, ttl} = Vault.Auth.Userpass.login(%{ username: username, password: password }) # String Map {:ok, token, ttl} = Vault.Auth.Userpass.login(%{ &quot;username&quot; =&gt; &quot;username&quot;, &quot;password&quot; =&gt; &quot;password&quot; })"},{"ref":"Vault.Engine.Adapter.html","title":"Vault.Engine.Adapter","type":"behaviour","doc":"Adapter specificication for Vault Secret Engines"},{"ref":"Vault.Engine.Adapter.html#c:delete/3","title":"Vault.Engine.Adapter.delete/3","type":"callback","doc":""},{"ref":"Vault.Engine.Adapter.html#c:list/3","title":"Vault.Engine.Adapter.list/3","type":"callback","doc":""},{"ref":"Vault.Engine.Adapter.html#c:read/3","title":"Vault.Engine.Adapter.read/3","type":"callback","doc":""},{"ref":"Vault.Engine.Adapter.html#c:write/4","title":"Vault.Engine.Adapter.write/4","type":"callback","doc":""},{"ref":"Vault.Engine.Adapter.html#t:data/0","title":"Vault.Engine.Adapter.data/0","type":"type","doc":""},{"ref":"Vault.Engine.Adapter.html#t:errors/0","title":"Vault.Engine.Adapter.errors/0","type":"type","doc":""},{"ref":"Vault.Engine.Adapter.html#t:options/0","title":"Vault.Engine.Adapter.options/0","type":"type","doc":""},{"ref":"Vault.Engine.Adapter.html#t:path/0","title":"Vault.Engine.Adapter.path/0","type":"type","doc":""},{"ref":"Vault.Engine.Adapter.html#t:response/0","title":"Vault.Engine.Adapter.response/0","type":"type","doc":""},{"ref":"Vault.Engine.Adapter.html#t:token/0","title":"Vault.Engine.Adapter.token/0","type":"type","doc":""},{"ref":"Vault.Engine.Adapter.html#t:value/0","title":"Vault.Engine.Adapter.value/0","type":"type","doc":""},{"ref":"Vault.Engine.Adapter.html#t:vault/0","title":"Vault.Engine.Adapter.vault/0","type":"type","doc":""},{"ref":"Vault.Engine.Generic.html","title":"Vault.Engine.Generic","type":"module","doc":"A generic Vault.Engine adapter. Most of the vault secret engines don&#39;t use a wildly different API, and can be handled with a single adapter. Request Details By default, read runs a GET request, write does a POST, list does a GET with an appended ?list=true, and delete runs a DELETE. The options below should give you additional flexibility. Request Options: :method - one of :get, :put, :post, :options, :patch, :head :full_response - if true, returns the full response body on success, rather than just the data key. Defaults to false, :query_params - query params for the request. Defaults to %{} (no params) :body - body to be sent along with the request. Defaults to %{} (no body) on read, or the passed in value on write Examples Create a generic vault client: {:ok, vault } = Vault.new( host: System.get_env(&quot;VAULT_ADDR&quot;), auth: Vault.Auth.Token, engine: Vault.Engine.Generic, http: Vault.HTTP.Tesla, ) |&gt; Vault.auth(%{token: &quot;token&quot;}) Read/Write from the cubbyhole secret engine. {:ok, _data} = Vault.write(vault, &quot;cubbyhole/hello&quot;, %{&quot;foo&quot; =&gt; &quot;bar&quot;}) {:ok, %{&quot;foo&quot; =&gt; &quot;bar&quot;}} = Vault.read(vault, &quot;cubbyhole/hello&quot;) Read/Write from the ssh secret engine. # create a key {:ok, _} = Vault.write(vault, &quot;ssh/keys/test&quot;, %{key: key}) # create a role for that key {:ok, _} = Vault.write(vault, &quot;ssh/roles/test&quot;, %{ key: &quot;test&quot;, key_type: &quot;dynamic&quot;, default_user: &quot;tester&quot;, admin_user: &quot;admin_tester&quot; }) # read a role, and return the full response {:ok, %{ &quot;data&quot; =&gt; data } } = Vault.read(vault, &quot;ssh-client-signer/roles/test&quot;, full_response: true) Options: :method - one of :get, :put, :post, :options, :patch, :head :full_response - if true, returns the full response body on success, rather than just the data key. Defaults to false, :params - query params for the request. Defaults to %{} (no params) :body - body to be sent along with the request. Defaults to %{} (no body) on read, or the passed in value on write"},{"ref":"Vault.Engine.Generic.html#list/3","title":"Vault.Engine.Generic.list/3","type":"function","doc":"Lists secrets at a path. Defaults to a GET request against the provided path, with a query param of ?list=true. See options details above for full configuration. Examples {:ok, %{ &quot;keys&quot;=&gt; [&quot;foo&quot;, &quot;foo/&quot;] } } = Vault.Engine.Generic.list(vault, &quot;path/to/list/&quot;, [full_response: true]) With the full Response: {:ok, %{ &quot;data&quot; =&gt; %{ &quot;keys&quot;=&gt; [&quot;foo&quot;, &quot;foo/&quot;] }, } } = Vault.Engine.Generic.list(vault, &quot;path/to/list/&quot;, [full_response: true])"},{"ref":"Vault.Engine.Generic.html#read/3","title":"Vault.Engine.Generic.read/3","type":"function","doc":"Gets a value from vault. Defaults to a GET request against the current path. See optionx details above for full configuration."},{"ref":"Vault.Engine.Generic.html#write/4","title":"Vault.Engine.Generic.write/4","type":"function","doc":"Puts a value in vault. Defaults to a POST request against the provided path.See options details above for full configuration."},{"ref":"Vault.Engine.Generic.html#t:errors/0","title":"Vault.Engine.Generic.errors/0","type":"type","doc":""},{"ref":"Vault.Engine.Generic.html#t:options/0","title":"Vault.Engine.Generic.options/0","type":"type","doc":""},{"ref":"Vault.Engine.Generic.html#t:path/0","title":"Vault.Engine.Generic.path/0","type":"type","doc":""},{"ref":"Vault.Engine.Generic.html#t:token/0","title":"Vault.Engine.Generic.token/0","type":"type","doc":""},{"ref":"Vault.Engine.Generic.html#t:value/0","title":"Vault.Engine.Generic.value/0","type":"type","doc":""},{"ref":"Vault.Engine.Generic.html#t:vault/0","title":"Vault.Engine.Generic.vault/0","type":"type","doc":""},{"ref":"Vault.Engine.KVV1.html","title":"Vault.Engine.KVV1","type":"module","doc":"Get and Put secrets using the v1 KV Secrets engine. See: Vault Docs for details."},{"ref":"Vault.Engine.KVV1.html#delete/3","title":"Vault.Engine.KVV1.delete/3","type":"function","doc":"Deletes secrets at a path"},{"ref":"Vault.Engine.KVV1.html#list/3","title":"Vault.Engine.KVV1.list/3","type":"function","doc":"Lists secrets at a path"},{"ref":"Vault.Engine.KVV1.html#read/3","title":"Vault.Engine.KVV1.read/3","type":"function","doc":"Gets a value from vault."},{"ref":"Vault.Engine.KVV1.html#write/4","title":"Vault.Engine.KVV1.write/4","type":"function","doc":"Puts a value in vault."},{"ref":"Vault.Engine.KVV2.html","title":"Vault.Engine.KVV2","type":"module","doc":"Get and put secrets using the v2 KV (versioned) secrets engine See: Vault Docs for details."},{"ref":"Vault.Engine.KVV2.html#delete/3","title":"Vault.Engine.KVV2.delete/3","type":"function","doc":"Soft or Hard Delete a versioned secret. Requires a list of versions to be removed. This request produces an empty body, so an empty map is returned. Examples Soft delete a version of a secret {:ok, %{ &quot;data&quot; =&gt; nil, &quot;metadata&quot; =&gt; %{ &quot;created_time&quot; =&gt; &quot;2018-11-21T19:49:49.339727561Z&quot;, &quot;deletion_time&quot; =&gt; &quot;2018-11-21T19:49:49.353904424Z&quot;, &quot;destroyed&quot; =&gt; false, &quot;version&quot; =&gt; 5 } } } = Vault.Engine.KVV2.Delete(vault, &quot;path/to/delete&quot;, versions: [5], full_response: true) Hard delete a secret {:ok, %{ &quot;data&quot; =&gt; nil, &quot;metadata&quot; =&gt; %{ &quot;created_time&quot; =&gt; &quot;2018-11-21T19:49:49.339727561Z&quot;, &quot;deletion_time&quot; =&gt; &quot;2018-11-21T19:49:49.353904424Z&quot;, &quot;destroyed&quot; =&gt; true, &quot;version&quot; =&gt; 5 } } } = Vault.Engine.KVV2.Delete(vault, &quot;path/to/delete&quot;, versions: [5], destroy: true, full_response: true)"},{"ref":"Vault.Engine.KVV2.html#list/3","title":"Vault.Engine.KVV2.list/3","type":"function","doc":"This endpoint returns a list of key names at the specified location. Folders are suffixed with /. The input must be a folder; list on a file will not return a value. Examples {:ok, %{ &quot;keys&quot;=&gt; [&quot;foo&quot;, &quot;foo/&quot;] } } = Vault.Engine.KVV2.List(vault, &quot;path/to/list/&quot;, [full_response: true]) With the full Response: {:ok, %{ &quot;data&quot; =&gt; %{ &quot;keys&quot;=&gt; [&quot;foo&quot;, &quot;foo/&quot;] }, } } = Vault.Engine.KVV2.List(vault, &quot;path/to/list/&quot;, [full_response: true])"},{"ref":"Vault.Engine.KVV2.html#read/3","title":"Vault.Engine.KVV2.read/3","type":"function","doc":"Get a secret from vault. Optionally supply a version, otherwise gets latest value. Examples Fetch a value at a specific version, with the :version option. {:ok, %{&quot;foo&quot; =&gt; &quot;bar&quot;}} = Vault.Engine.KVV2.read(vault, &quot;secret/to/get, [version: 1]) {:ok, %{&quot;bar&quot; =&gt; &quot;baz&quot;}} = Vault.Engine.KVV2.read(vault, &quot;secret/to/get, [version: 2]) Because of the nature of soft deletes,fetching soft-deleted secrets will return an error. {:error, [&quot;Key not found&quot;]} = Vault.Engine.KVV2.read(vault, &quot;soft/deleted/secret&quot;, [version: 1]) However, if you wish to see the metadata or additional values, setting full_response to true will return can return a soft deleted key as a success. {:ok, %{ &quot;auth&quot; =&gt; nil, &quot;data&quot; =&gt; %{ &quot;data&quot; =&gt; nil, &quot;metadata&quot; =&gt; %{ &quot;created_time&quot; =&gt; &quot;2018-11-21T19:49:49.339727561Z&quot;, &quot;deletion_time&quot; =&gt; &quot;2018-11-21T19:49:49.353904424Z&quot;, &quot;destroyed&quot; =&gt; false, &quot;version&quot; =&gt; 1 } }, &quot;lease_duration&quot; =&gt; 0, &quot;lease_id&quot; =&gt; &quot;&quot;, &quot;renewable&quot; =&gt; false, &quot;request_id&quot; =&gt; &quot;e289ff31-609f-44fa-7161-55c63fda3d43&quot;, &quot;warnings&quot; =&gt; nil, &quot;wrap_info&quot; =&gt; nil } } = Vault.Engine.KVV2.read(vault, &quot;soft/deleted/secret&quot;, [version: 1, full_response: true]) Options: version: integer - the version you want to return. full_response: boolean - get the whole reponse back on success, not just the data field"},{"ref":"Vault.Engine.KVV2.html#write/4","title":"Vault.Engine.KVV2.write/4","type":"function","doc":"Put a secret in vault, on a given path. Examples Write a new version: {:ok, %{}} = Vault.Engine.Generic.write(vault, &quot;path/to/write&quot;, %{ foo: &quot;bar&quot; }) Check and set - see Vault Docs for details # write only if the value doesn&#39;t exist {:ok, response } = Vault.Engine.Generic.write(vault, &quot;path/to/write&quot;, %{ foo: &quot;bar&quot; }, [cas: 0]) # write only if the cas matches the current secret version {:ok, response } = Vault.Engine.Generic.write(vault, &quot;path/to/write&quot;, %{ foo: &quot;bar&quot; }, [cas: 1]) Get the full response body from vault: {:ok, %{ &quot;data&quot; =&gt; %{ &quot;created_time&quot; =&gt; &quot;2018-03-22T02:36:43.986212308Z&quot;, &quot;deletion_time&quot; =&gt; &quot;&quot;, &quot;destroyed&quot; =&gt; false, &quot;version&quot; =&gt; 1 }, } } = Vault.Engine.Generic.write(vault, &quot;path/to/write&quot;, %{ foo: &quot;bar&quot; }, [full_response: true]) Options cas: integer set a check-and-set value full_response: boolean - get the whole reponse back on success, not just the data field"},{"ref":"Vault.Engine.KVV2.html#t:errors/0","title":"Vault.Engine.KVV2.errors/0","type":"type","doc":""},{"ref":"Vault.Engine.KVV2.html#t:options/0","title":"Vault.Engine.KVV2.options/0","type":"type","doc":""},{"ref":"Vault.Engine.KVV2.html#t:path/0","title":"Vault.Engine.KVV2.path/0","type":"type","doc":""},{"ref":"Vault.Engine.KVV2.html#t:token/0","title":"Vault.Engine.KVV2.token/0","type":"type","doc":""},{"ref":"Vault.Engine.KVV2.html#t:value/0","title":"Vault.Engine.KVV2.value/0","type":"type","doc":""},{"ref":"Vault.Engine.KVV2.html#t:vault/0","title":"Vault.Engine.KVV2.vault/0","type":"type","doc":""},{"ref":"Vault.Engine.KVV2.html#t:version/0","title":"Vault.Engine.KVV2.version/0","type":"type","doc":""},{"ref":"Vault.HTTP.html","title":"Vault.HTTP","type":"module","doc":"Module that ultimately creates, formats, and parses all http requests and responses for each vault API call. Requests can take the following options a Keyword List. options: :query_params - List of query params for the request. Do not include query params on the path. :body - The JSON body for the request. :headers - List of headers for the request :version - The vault api version - defaults to &quot;v1&quot;"},{"ref":"Vault.HTTP.html#delete/3","title":"Vault.HTTP.delete/3","type":"function","doc":"Make a DELETE request against the configured vault instance. See options above for configuration."},{"ref":"Vault.HTTP.html#get/3","title":"Vault.HTTP.get/3","type":"function","doc":"Make a GET request against the configured vault instance. See options above for configuration."},{"ref":"Vault.HTTP.html#head/3","title":"Vault.HTTP.head/3","type":"function","doc":"Make a HEAD request against the configured vault instance. See options above for configuration."},{"ref":"Vault.HTTP.html#patch/3","title":"Vault.HTTP.patch/3","type":"function","doc":"Make a PATCH request against the configured vault instance. See options above for configuration."},{"ref":"Vault.HTTP.html#post/3","title":"Vault.HTTP.post/3","type":"function","doc":"Make a POST request against the configured vault instance. See options above for configuration."},{"ref":"Vault.HTTP.html#put/3","title":"Vault.HTTP.put/3","type":"function","doc":"Make a PUT request against the configured vault instance. See options above for configuration."},{"ref":"Vault.HTTP.html#request/4","title":"Vault.HTTP.request/4","type":"function","doc":"Make an arbitrary request against the configured vault instance. See options above for configuration."},{"ref":"Vault.HTTP.html#t:body/0","title":"Vault.HTTP.body/0","type":"type","doc":""},{"ref":"Vault.HTTP.html#t:headers/0","title":"Vault.HTTP.headers/0","type":"type","doc":""},{"ref":"Vault.HTTP.html#t:path/0","title":"Vault.HTTP.path/0","type":"type","doc":""},{"ref":"Vault.HTTP.html#t:query_params/0","title":"Vault.HTTP.query_params/0","type":"type","doc":""},{"ref":"Vault.HTTP.html#t:version/0","title":"Vault.HTTP.version/0","type":"type","doc":""},{"ref":"Vault.HTTP.Adapter.html","title":"Vault.HTTP.Adapter","type":"behaviour","doc":"Adapter interface for making Vault HTTP. Vault comes with a basic Tesla Adapter, providing support for hackney, httpc, and ibrowse"},{"ref":"Vault.HTTP.Adapter.html#c:request/5","title":"Vault.HTTP.Adapter.request/5","type":"callback","doc":""},{"ref":"Vault.HTTP.Adapter.html#t:headers/0","title":"Vault.HTTP.Adapter.headers/0","type":"type","doc":""},{"ref":"Vault.HTTP.Adapter.html#t:http_options/0","title":"Vault.HTTP.Adapter.http_options/0","type":"type","doc":""},{"ref":"Vault.HTTP.Adapter.html#t:method/0","title":"Vault.HTTP.Adapter.method/0","type":"type","doc":""},{"ref":"Vault.HTTP.Adapter.html#t:params/0","title":"Vault.HTTP.Adapter.params/0","type":"type","doc":""},{"ref":"Vault.HTTP.Adapter.html#t:response/0","title":"Vault.HTTP.Adapter.response/0","type":"type","doc":""},{"ref":"Vault.HTTP.Adapter.html#t:url/0","title":"Vault.HTTP.Adapter.url/0","type":"type","doc":""},{"ref":"Vault.HTTP.Tesla.html","title":"Vault.HTTP.Tesla","type":"module","doc":"Tesla HTTP Adapter for Vault.Http calls. It uses the following middleware: Tesla.Middleware.FollowRedirects Setup Add tesla and your http client of choice to your dependacies. defp deps() do [ # ... your other deps {:tesla, &quot;~&gt; 1.2.0&quot;}, {:hackney, &quot;~&gt; 1.10&quot;} # or :ibrowse, if that&#39;s your jam ] end Configuring Tesla: You can configure which client you want to use with tesla by adding the following to your mix config: config :tesla, adapter: Tesla.Adapters.Hackney # or adapter: Tesla.Adapters.IBrowse You can also use httpc, but be aware that there&#39;s some strange behavior around httpc redirects at this time. Passing HTTP options You may want to configure and pass HTTP options to the adapter you choose. In this case, you can pass the http_options right to the Vault client, like: Vault.new([ engine: Vault.Engine.KVV2, http_options: [ adapter: {Tesla.Adapter.Hackney, ssl_options: [cacertfile: &quot;/path/to/file&quot;]}, middleware: [ {Tesla.Middleware.Retry, delay: 500, max_retries: 10}, {Tesla.Middleware.Logger, log_level: :debug}, {Tesla.Middleware.FollowRedirects, max_redirects: 3} ] ] ])"},{"ref":"Vault.HTTP.Tesla.html#delete/3","title":"Vault.HTTP.Tesla.delete/3","type":"function","doc":"Perform a DELETE request. See request/1 or request/2 for options definition. delete(&quot;/users&quot;) delete(&quot;/users&quot;, query: [scope: &quot;admin&quot;]) delete(client, &quot;/users&quot;) delete(client, &quot;/users&quot;, query: [scope: &quot;admin&quot;])"},{"ref":"Vault.HTTP.Tesla.html#delete!/3","title":"Vault.HTTP.Tesla.delete!/3","type":"function","doc":"Perform a DELETE request. See request!/1 or request!/2 for options definition. delete!(&quot;/users&quot;) delete!(&quot;/users&quot;, query: [scope: &quot;admin&quot;]) delete!(client, &quot;/users&quot;) delete!(client, &quot;/users&quot;, query: [scope: &quot;admin&quot;])"},{"ref":"Vault.HTTP.Tesla.html#get/3","title":"Vault.HTTP.Tesla.get/3","type":"function","doc":"Perform a GET request. See request/1 or request/2 for options definition. get(&quot;/users&quot;) get(&quot;/users&quot;, query: [scope: &quot;admin&quot;]) get(client, &quot;/users&quot;) get(client, &quot;/users&quot;, query: [scope: &quot;admin&quot;])"},{"ref":"Vault.HTTP.Tesla.html#get!/3","title":"Vault.HTTP.Tesla.get!/3","type":"function","doc":"Perform a GET request. See request!/1 or request!/2 for options definition. get!(&quot;/users&quot;) get!(&quot;/users&quot;, query: [scope: &quot;admin&quot;]) get!(client, &quot;/users&quot;) get!(client, &quot;/users&quot;, query: [scope: &quot;admin&quot;])"},{"ref":"Vault.HTTP.Tesla.html#head/3","title":"Vault.HTTP.Tesla.head/3","type":"function","doc":"Perform a HEAD request. See request/1 or request/2 for options definition. head(&quot;/users&quot;) head(&quot;/users&quot;, query: [scope: &quot;admin&quot;]) head(client, &quot;/users&quot;) head(client, &quot;/users&quot;, query: [scope: &quot;admin&quot;])"},{"ref":"Vault.HTTP.Tesla.html#head!/3","title":"Vault.HTTP.Tesla.head!/3","type":"function","doc":"Perform a HEAD request. See request!/1 or request!/2 for options definition. head!(&quot;/users&quot;) head!(&quot;/users&quot;, query: [scope: &quot;admin&quot;]) head!(client, &quot;/users&quot;) head!(client, &quot;/users&quot;, query: [scope: &quot;admin&quot;])"},{"ref":"Vault.HTTP.Tesla.html#options/3","title":"Vault.HTTP.Tesla.options/3","type":"function","doc":"Perform a OPTIONS request. See request/1 or request/2 for options definition. options(&quot;/users&quot;) options(&quot;/users&quot;, query: [scope: &quot;admin&quot;]) options(client, &quot;/users&quot;) options(client, &quot;/users&quot;, query: [scope: &quot;admin&quot;])"},{"ref":"Vault.HTTP.Tesla.html#options!/3","title":"Vault.HTTP.Tesla.options!/3","type":"function","doc":"Perform a OPTIONS request. See request!/1 or request!/2 for options definition. options!(&quot;/users&quot;) options!(&quot;/users&quot;, query: [scope: &quot;admin&quot;]) options!(client, &quot;/users&quot;) options!(client, &quot;/users&quot;, query: [scope: &quot;admin&quot;])"},{"ref":"Vault.HTTP.Tesla.html#patch/4","title":"Vault.HTTP.Tesla.patch/4","type":"function","doc":"Perform a PATCH request. See request/1 or request/2 for options definition. patch(&quot;/users&quot;, %{name: &quot;Jon&quot;}) patch(&quot;/users&quot;, %{name: &quot;Jon&quot;}, query: [scope: &quot;admin&quot;]) patch(client, &quot;/users&quot;, %{name: &quot;Jon&quot;}) patch(client, &quot;/users&quot;, %{name: &quot;Jon&quot;}, query: [scope: &quot;admin&quot;])"},{"ref":"Vault.HTTP.Tesla.html#patch!/4","title":"Vault.HTTP.Tesla.patch!/4","type":"function","doc":"Perform a PATCH request. See request!/1 or request!/2 for options definition. patch!(&quot;/users&quot;, %{name: &quot;Jon&quot;}) patch!(&quot;/users&quot;, %{name: &quot;Jon&quot;}, query: [scope: &quot;admin&quot;]) patch!(client, &quot;/users&quot;, %{name: &quot;Jon&quot;}) patch!(client, &quot;/users&quot;, %{name: &quot;Jon&quot;}, query: [scope: &quot;admin&quot;])"},{"ref":"Vault.HTTP.Tesla.html#post/4","title":"Vault.HTTP.Tesla.post/4","type":"function","doc":"Perform a POST request. See request/1 or request/2 for options definition. post(&quot;/users&quot;, %{name: &quot;Jon&quot;}) post(&quot;/users&quot;, %{name: &quot;Jon&quot;}, query: [scope: &quot;admin&quot;]) post(client, &quot;/users&quot;, %{name: &quot;Jon&quot;}) post(client, &quot;/users&quot;, %{name: &quot;Jon&quot;}, query: [scope: &quot;admin&quot;])"},{"ref":"Vault.HTTP.Tesla.html#post!/4","title":"Vault.HTTP.Tesla.post!/4","type":"function","doc":"Perform a POST request. See request!/1 or request!/2 for options definition. post!(&quot;/users&quot;, %{name: &quot;Jon&quot;}) post!(&quot;/users&quot;, %{name: &quot;Jon&quot;}, query: [scope: &quot;admin&quot;]) post!(client, &quot;/users&quot;, %{name: &quot;Jon&quot;}) post!(client, &quot;/users&quot;, %{name: &quot;Jon&quot;}, query: [scope: &quot;admin&quot;])"},{"ref":"Vault.HTTP.Tesla.html#put/4","title":"Vault.HTTP.Tesla.put/4","type":"function","doc":"Perform a PUT request. See request/1 or request/2 for options definition. put(&quot;/users&quot;, %{name: &quot;Jon&quot;}) put(&quot;/users&quot;, %{name: &quot;Jon&quot;}, query: [scope: &quot;admin&quot;]) put(client, &quot;/users&quot;, %{name: &quot;Jon&quot;}) put(client, &quot;/users&quot;, %{name: &quot;Jon&quot;}, query: [scope: &quot;admin&quot;])"},{"ref":"Vault.HTTP.Tesla.html#put!/4","title":"Vault.HTTP.Tesla.put!/4","type":"function","doc":"Perform a PUT request. See request!/1 or request!/2 for options definition. put!(&quot;/users&quot;, %{name: &quot;Jon&quot;}) put!(&quot;/users&quot;, %{name: &quot;Jon&quot;}, query: [scope: &quot;admin&quot;]) put!(client, &quot;/users&quot;, %{name: &quot;Jon&quot;}) put!(client, &quot;/users&quot;, %{name: &quot;Jon&quot;}, query: [scope: &quot;admin&quot;])"},{"ref":"Vault.HTTP.Tesla.html#request/2","title":"Vault.HTTP.Tesla.request/2","type":"function","doc":"Perform a request. Options :method - the request method, one of [:head, :get, :delete, :trace, :options, :post, :put, :patch] :url - either full url e.g. &quot;http://example.com/some/path&quot; or just &quot;/some/path&quot; if using Tesla.Middleware.BaseUrl :query - a keyword list of query params, e.g. [page: 1, per_page: 100] :headers - a keyworld list of headers, e.g. [{&quot;content-type&quot;, &quot;text/plain&quot;}] :body - depends on used middleware: by default it can be a binary if using e.g. JSON encoding middleware it can be a nested map if adapter supports it it can be a Stream with any of the above :opts - custom, per-request middleware or adapter options Examples ExampleApi.request(method: :get, url: &quot;/users/path&quot;) # use shortcut methods ExampleApi.get(&quot;/users/1&quot;) ExampleApi.post(client, &quot;/users&quot;, %{name: &quot;Jon&quot;})"},{"ref":"Vault.HTTP.Tesla.html#request!/2","title":"Vault.HTTP.Tesla.request!/2","type":"function","doc":"Perform request and raise in case of error. This is similar to request/2 behaviour from Tesla 0.x See request/2 for list of available options."},{"ref":"Vault.HTTP.Tesla.html#trace/3","title":"Vault.HTTP.Tesla.trace/3","type":"function","doc":"Perform a TRACE request. See request/1 or request/2 for options definition. trace(&quot;/users&quot;) trace(&quot;/users&quot;, query: [scope: &quot;admin&quot;]) trace(client, &quot;/users&quot;) trace(client, &quot;/users&quot;, query: [scope: &quot;admin&quot;])"},{"ref":"Vault.HTTP.Tesla.html#trace!/3","title":"Vault.HTTP.Tesla.trace!/3","type":"function","doc":"Perform a TRACE request. See request!/1 or request!/2 for options definition. trace!(&quot;/users&quot;) trace!(&quot;/users&quot;, query: [scope: &quot;admin&quot;]) trace!(client, &quot;/users&quot;) trace!(client, &quot;/users&quot;, query: [scope: &quot;admin&quot;])"},{"ref":"Vault.HTTP.Tesla.html#t:option/0","title":"Vault.HTTP.Tesla.option/0","type":"type","doc":""},{"ref":"Vault.JSON.Adapter.html","title":"Vault.JSON.Adapter","type":"behaviour","doc":"The adapter interface for encoding, and decoding json or vault requests Recommended JSON adapters: Jason Poison"},{"ref":"Vault.JSON.Adapter.html#c:decode/2","title":"Vault.JSON.Adapter.decode/2","type":"callback","doc":""},{"ref":"Vault.JSON.Adapter.html#c:decode!/2","title":"Vault.JSON.Adapter.decode!/2","type":"callback","doc":""},{"ref":"Vault.JSON.Adapter.html#c:encode/2","title":"Vault.JSON.Adapter.encode/2","type":"callback","doc":""},{"ref":"Vault.JSON.Adapter.html#c:encode!/2","title":"Vault.JSON.Adapter.encode!/2","type":"callback","doc":""},{"ref":"Vault.JSON.Adapter.html#t:options/0","title":"Vault.JSON.Adapter.options/0","type":"type","doc":""},{"ref":"readme.html","title":"libvault","type":"extras","doc":"libvault Highly configurable library for HashiCorp&#39;s Vault - handles authentication for multiple backends, and reading, writing, listing, and deleting secrets for a variety of engines. When possible, it tries to emulate the CLI, with read, write, list and delete and auth methods. An additional request method is provided when you need further flexibility with the API."},{"ref":"readme.html#api-preview","title":"libvault - API Preview","type":"extras","doc":"{:ok, vault } = Vault.new([ engine: Vault.Engine.KVV2, auth: Vault.Auth.UserPass ]) |&gt; Vault.auth(%{username: &quot;username&quot;, password: &quot;password&quot;}) {:ok, db_pass} = Vault.read(vault, &quot;secret/path/to/password&quot;) {:ok, %{&quot;version&quot; =&gt; 1 }} = Vault.write(vault, &quot;secret/path/to/creds&quot;, %{secret: &quot;secrets!&quot;})"},{"ref":"readme.html#configuration-adapters","title":"libvault - Configuration / Adapters","type":"extras","doc":"Hashicorp&#39;s Vault is highly configurable. Rather than cover every possible option, this library strives to be flexible and adaptable. Auth backends, Secret Engines, and HTTP clients are all replaceable, and each behaviour asks for a minimal contract."},{"ref":"readme.html#http-adapters","title":"libvault - HTTP Adapters","type":"extras","doc":"The following HTTP Adapters are provided: Tesla with Vault.HTTP.Tesla Can be configured to use Hackney, iBrowse or Mint Be sure to add applications and dependencies to your mix file as needed. JSON Adapters Most JSON libraries provide the same methods, so no default adapter is needed. You can use Jason, JSX, Poison, or whatever encoder you want. Defaults to Jason or Poison if present. See Vault.JSON.Adapter for the full behaviour interface."},{"ref":"readme.html#auth-adapters","title":"libvault - Auth Adapters","type":"extras","doc":"Adapters have been provided for the following auth backends: AppRole with Vault.Auth.Approle Azure with Vault.Auth.Azure GitHub with Vault.Auth.Github GoogleCloud with with Vault.Auth.GoogleCloud JWT with Vault.Auth.JWT Kubernetes with Vault.Auth.Kubernetes LDAP with Vault.Auth.LDAP UserPass with Vault.Auth.UserPass Token with Vault.Auth.Token In addition to the above, a generic backend is also provided (Vault.Auth.Generic). If support for auth provider is missing, you can still get up and running quickly, without writing a new adapter."},{"ref":"readme.html#secret-engine-adapters","title":"libvault - Secret Engine Adapters","type":"extras","doc":"Most of Vault&#39;s Secret Engines use a replacable API. The Vault.Engine.Generic adapter should handle most use cases for secret fetching. Vault&#39;s KV version 2 broke away from the standard REST convention. So KV has been given its own adapter: Key/Value v1 with Vault.Engine.KVV1 v2 with Vault.Engine.KVV2 Additional request methods The core library only handles the basics around secret fetching. If you need to access additional API endpoints, this library also provides a Vault.request method. This should allow you to tap into the complete vault REST API, while still benefiting from token control, JSON parsing, and other HTTP client nicities."},{"ref":"readme.html#installation-and-usage","title":"libvault - Installation and Usage","type":"extras","doc":"Installation Ensure that any adapter dependencies have been included as part of your application&#39;s dependencies: def deps do [ {:libvault, &quot;~&gt; 0.2.0&quot;}, # tesla, required for Vault.HTTP.Tesla {:tesla, &quot;~&gt; 1.3.0&quot;}, # pick your HTTP client - Mint, iBrowse or hackney {:mint, &quot;~&gt; 0.4.0&quot;}, {:castore, &quot;~&gt; 0.1.0&quot;}, # Pick your json parser - Jason or Poison {:jason, &quot;&gt;= 1.0.0&quot;} ] end Usage vault = Vault.new([ engine: Vault.Engine.KVV2, auth: Vault.Auth.UserPass, json: Jason, credentials: %{username: &quot;username&quot;, password: &quot;password&quot;} ]) |&gt; Vault.auth() {:ok, db_pass} = Vault.read(vault, &quot;secret/path/to/password&quot;) {:ok, %{&quot;version&quot; =&gt; 1 }} = Vault.write(vault, &quot;secret/path/to/creds&quot;, %{secret: &quot;secrets!&quot;}) You can configure the vault client up front, or change configuration on the fly. vault = Vault.new() |&gt; Vault.set_auth(Vault.Auth.Approle) |&gt; Vault.set_engine(Vault.Engine.Generic) |&gt; Vault.auth(%{role_id: &quot;role_id&quot;, secret_id: &quot;secret_id&quot;}) {:ok, db_pass} = Vault.read(vault, &quot;secret/path/to/password&quot;) vault = Vault.set_engine(Vault.Engine.KVV2) // switch to versioned secrets {:ok, db_pass} = Vault.write(vault, &quot;kv/path/to/password&quot;, %{ password: &quot;db_pass&quot; }) See the full Vault client for additional methods."},{"ref":"readme.html#testing-locally","title":"libvault - Testing Locally","type":"extras","doc":"When possible, tests run against a local vault instance. Otherwise, tests run against the Vault Spec, using bypass to test to confirm the success case, and follows vault patterns for failure. Install the Vault Go CLI https://www.vaultproject.io/downloads.html In the current directory, set up a local dev server with sh scripts/setup-local-vault Vault (at this time) can&#39;t be run in the background without a docker instance. For now, set up the local secret engine paths with sh scripts/setup-engines.sh Documentation can be generated with ExDoc and published on HexDocs. Once published, the docs can be found at https://hexdocs.pm/libvault."}]